// ============================================================================= //
// Copyright 2007-2023 Jrgen Srensen - www.jososoft.dk - js@jososoft.dk ====== //   
// ============================================================================= //

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;
import javax.swing.*;
import javax.swing.filechooser.*;

@SuppressWarnings("serial")
public class VolumeChanger extends JFrame implements ActionListener
{
	private String copyright = " 2007-2023 Jrgen Srensen";
	private String email = "js@jososoft.dk";
	private String www = "http://www.jososoft.dk/yamaha/software/volchange";
	private String ver = "1.10";
	private String app_name = "Style Volume Changer";

	protected static String sep = System.getProperty("file.separator");
	private static File logDir0 = new File(System.getProperty("user.home"), "Application Data");
	private static File logDir = new File(System.getProperty("user.home"), "Application Data"  + sep + "jososoft" );

	private double view = 1.0D;
	private double view_ini = 1.0D;
	private double step;
	private boolean set_once = false;
	private Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

	private JMenuBar bar;
	private JMenu filerMenu, viewMenu, helpMenu;
	private JMenuItem open, openD, exit, w0, w1, w2, w3, w4, w5, onlineHelp, help, tech_help, regis, about, checkNew;

	private JButton go, exitB;
	static JComboBox<String> perc;
	static JCheckBox vol, vel;
	static JCheckBox up, down;

	protected static String convertDir, saveDir;
	File stylefile;
	File[] files;
	static byte[] content; 
	static int value;
	static int m_end;
	static String filename;
	static Sequence newStyle; 
	static int mode;

	public static void main(String[] args) 
	{
    	VolumeChanger runprogram = new VolumeChanger();
  	}

	VolumeChanger()
	{
		super("Style Volume Changer");
		this.addWindowListener(
			new WindowAdapter()
			{	public void windowClosing(WindowEvent e)
				{
					save();
					System.exit(0);
				}
			}
		);

		try
		{
			if (!logDir0.exists())
				logDir0.mkdir();
			if (!logDir.exists())
				logDir.mkdir();
			File temp1 = new File(logDir, "StyleVolumeChanger.ini"); // <<=========================== 
			BufferedReader bdr = new BufferedReader(new FileReader(temp1.getAbsolutePath()));
			String inputLine;
			int i = 0;
			do
			{
				inputLine = bdr.readLine();
				if (inputLine == null) 
					break;
				if (i == 0)
					view_ini = Double.parseDouble(inputLine); 
				i++;
			}
			while(inputLine != null);
			bdr.close();
			bdr = null;
		}
		catch(Exception e)
		{}

		bar = new JMenuBar();
		setJMenuBar(bar);

		filerMenu = new JMenu("File");
		open = new JMenuItem("Open Style File");
		open.setAccelerator(KeyStroke.getKeyStroke('O', ActionEvent.CTRL_MASK));
		open.addActionListener(this);
		filerMenu.add(open);
		openD = new JMenuItem("Open Folder");
		openD.setAccelerator(KeyStroke.getKeyStroke('F', ActionEvent.CTRL_MASK));
		openD.addActionListener(this);
		filerMenu.add(openD);
		filerMenu.addSeparator();
		exit = new JMenuItem("Exit");
		exit.setAccelerator(KeyStroke.getKeyStroke('X', ActionEvent.CTRL_MASK));
		exit.addActionListener(this);
		filerMenu.add(exit);
		bar.add(filerMenu);

		viewMenu = new JMenu("View");
		w0 = new JMenuItem("Normal");
		w0.setAccelerator(KeyStroke.getKeyStroke('0', ActionEvent.CTRL_MASK));
		w0.addActionListener(this);
		viewMenu.add(w0);
		w1 = new JMenuItem("Large");
		w1.setAccelerator(KeyStroke.getKeyStroke('1', ActionEvent.CTRL_MASK));
		w1.addActionListener(this);
		viewMenu.add(w1);
		w2 = new JMenuItem("XL");
		w2.setAccelerator(KeyStroke.getKeyStroke('2', ActionEvent.CTRL_MASK));
		w2.addActionListener(this);
		viewMenu.add(w2);
		w3 = new JMenuItem("XXL");
		w3.setAccelerator(KeyStroke.getKeyStroke('3', ActionEvent.CTRL_MASK));
		w3.addActionListener(this);
		viewMenu.add(w3);
		w4 = new JMenuItem("XXXL");
		w4.setAccelerator(KeyStroke.getKeyStroke('4', ActionEvent.CTRL_MASK));
		w4.addActionListener(this);
		viewMenu.add(w4);
		w5 = new JMenuItem("XXXXL");
		w5.setAccelerator(KeyStroke.getKeyStroke('5', ActionEvent.CTRL_MASK));
		w5.addActionListener(this);
		viewMenu.add(w5);
		bar.add(viewMenu);

		helpMenu = new JMenu("Help");
		onlineHelp = new JMenuItem("Online Help");
		onlineHelp.setAccelerator(KeyStroke.getKeyStroke('W', ActionEvent.CTRL_MASK));
		onlineHelp.addActionListener(this);
		helpMenu.add(onlineHelp);
		help = new JMenuItem("Help");
		help.setAccelerator(KeyStroke.getKeyStroke('H', ActionEvent.CTRL_MASK));
		help.addActionListener(this);
		helpMenu.add(help);
		tech_help = new JMenuItem("Technical Help");
		tech_help.addActionListener(this);
		helpMenu.add(tech_help);
		helpMenu.addSeparator();
		checkNew = new JMenuItem("Check for new Version");
		checkNew.addActionListener(this);
		helpMenu.add(checkNew);
		regis = new JMenuItem("Registration");
		regis.setAccelerator(KeyStroke.getKeyStroke('R', ActionEvent.CTRL_MASK));
		regis.addActionListener(this);
		helpMenu.add(regis);
		about = new JMenuItem("About");
		about.setAccelerator(KeyStroke.getKeyStroke('A', ActionEvent.CTRL_MASK));
		about.addActionListener(this);
		helpMenu.add(about);
		bar.add(helpMenu);

		setLayout(new GridLayout(3,1));

		JPanel p1 = new JPanel();
		p1.setLayout(new GridLayout(1,1));
		vol = new JCheckBox("Changes Volume in all channels");
		vol.addActionListener(this);
		vol.setSelected(true);
		p1.add(vol);
		add(p1);

		JPanel p2 = new JPanel();
		p2.setLayout(new GridLayout(1,1));
		vel = new JCheckBox("Changes Velocity at all Note On's");
		vel.addActionListener(this);
		p2.add(vel);
		add(p2);

		JPanel p3 = new JPanel();
		p3.setLayout(new GridLayout(1,6));
		perc = new JComboBox<>();
		for (int i = 0; i < 51; i++)
			perc.addItem("" + i + " %");
		perc.setEditable(false);
		p3.add(perc);
		up = new JCheckBox("Up");
		up.addActionListener(this);
		up.setSelected(true);
		p3.add(up);
		down = new JCheckBox("Down");
		down.addActionListener(this);
		p3.add(down);
		go = new JButton("Go !");
		go.addActionListener(this);
		p3.add(go);
		p3.add(new JLabel(""));
		exitB = new JButton("Exit");
		exitB.addActionListener(this);
		p3.add(exitB);
		add(p3);

		Image icon = Toolkit.getDefaultToolkit().getImage("icon.gif");
		setIconImage(icon); 
		setResizable(false);
		pack();

        double step_w = Math.pow(((double)d.width / (double)getWidth()), 0.2);		//5 steps
        double step_h = Math.pow(((double)d.height / (double)getHeight()), 0.2);

        if (step_w > step_h)
            step = step_h;
        else
            step = step_w;

        // reduce UI to double size in step 5
        if (step > 1.148)
            step = 1.148;
		
		resizeUI(view_ini, this); // <<=========================== indeholder setLocation()
		setVisible(true);
	}

	public void resizeUI(double factor, Container container)
	{		
		if (!set_once)
		{
			view = view * factor;
			for (int i = 0; i < bar.getMenuCount(); i++)
			{
				JMenu m = bar.getMenu(i);
				for (int j = 0; j < m.getMenuComponentCount(); j++)
				{
					JMenuItem mm = m.getItem(j);
					try	//avoid separators
					{
						Font fc = mm.getFont();
						int ft = fc.getSize();
						int fs = fc.getStyle();
						fc = new Font(null, fs, (int)(factor * ft + 0.5));
						mm.setFont(fc);
					}
					catch (Exception e)
					{
						//e.printStackTrace();
					}
				}
			}
			set_once = true;
		}

		Component[] componentArray;
		componentArray = container.getComponents();

		Dimension d1 = container.getSize();
		d1.height = (int)(factor * d1.height + 0.5);
		d1.width = (int)(factor * d1.width + 0.5);
		container.setSize(d1);

		for (int i=0; i < componentArray.length;i++)
		{
			try
			{
				Font fd = componentArray[i].getFont();
				int ft = fd.getSize();
				int fs = fd.getStyle();
				fd = new Font(null, fs, (int)(factor * ft + 0.5));
				componentArray[i].setFont(fd);
			}
			catch (Exception e)
			{}

			if (componentArray[i] instanceof Container)
			{
				resizeUI(factor, (Container)componentArray[i]);
			}
			else
			{
				Dimension d2 = componentArray[i].getSize();
				d2.height = (int)(factor * d2.height + 0.5);
				d2.width = (int)(factor * d2.width + 0.5);
				componentArray[i].setSize(d2);
			}
		}
		setLocation((d.width/2) - (getWidth()/2), (d.height/2) - (getHeight()/2));
	}

	private void save()
	{
		try
		{
			File temp1 = new File(logDir, "StyleVolumeChanger.ini");
			BufferedWriter out = new BufferedWriter(new FileWriter(temp1.getAbsolutePath(), false));
			out.write("" + view);	
			out.close();
			out = null;
		}
		catch (IOException ioe)
		{}
	}	

	public void actionPerformed(ActionEvent ae)
	{
		if (ae.getSource() == exit || ae.getSource() == exitB)
		{
			save();
			System.exit(0);
		}
		if (ae.getSource() == up)
		{
			if (up.isSelected())
				down.setSelected(false);
			else if (down.isSelected())
				up.setSelected(false);
		}
		else if (ae.getSource() == down)
		{
			if (down.isSelected())
				up.setSelected(false);
			else if (up.isSelected())
				down.setSelected(false);
		}
		else if (ae.getSource() == vel)
		{
			if (vel.isSelected())
				vol.setSelected(false);
			else if (vol.isSelected())
				vel.setSelected(false);
		}
		else if (ae.getSource() == vol)
		{
			if (vol.isSelected())
				vel.setSelected(false);
			else if (vel.isSelected())
				vol.setSelected(false);
		}

		else if (ae.getSource() == w0)
		{
			set_once = false;
			resizeUI(1.0 / view , this);
		}
		else if (ae.getSource() == w1)
		{
			set_once = false;
			resizeUI(1.0 * step / view , this);
		}
		else if (ae.getSource() == w2)
		{
			set_once = false;
			resizeUI(1.0 * step * step / view , this);
		}
		else if (ae.getSource() == w3)
		{
			set_once = false;
			resizeUI(1.0 * step * step * step / view , this);
		}
		else if (ae.getSource() == w4)
		{
			set_once = false;
			resizeUI(1.0 * step * step * step * step / view , this);
		}
		else if (ae.getSource() == w5)
		{
			set_once = false;
			resizeUI(1.0 * step * step * step * step * step / view , this);
		}
		else if (ae.getSource() == onlineHelp)
		{
			Browse br = new Browse(www + "/help.htm");
		}
		else if (ae.getSource() == checkNew)
		{
			Browse br = new Browse(www + "/index.htm#" + ver);
		}
		else if (ae.getSource() == help)
		{
			if (!Help.aktiv)
			{
				Help hlp = new Help("readme.txt", www + "/help.htm");
			}
		}
		else if (ae.getSource() == tech_help)
		{
			Browse br = new Browse("http://www.jososoft.dk/yamaha/articles/software_5.htm");
		}
		else if (ae.getSource() == regis)
		{
			if (!Regis.aktiv)
			{
				Regis rg = new Regis();
			}
		}
		else if (ae.getSource() == about)
		{
			About ab = new About(copyright, email, www, ver, app_name);
		}
		else if (ae.getSource() == open)
		{
			mode = 0;
			go.setEnabled(false);
			try
			{
				JFileChooser Jfidi = new JFileChooser("");
				Jfidi.setDialogTitle("Open Style File(s)");
				FileNameExtensionFilter filter = new FileNameExtensionFilter("Style Files", XG.validExtensions);
    			Jfidi.setFileFilter(filter);
				Jfidi.setMultiSelectionEnabled(true);

				int JreturnValue = Jfidi.showOpenDialog(new JFrame());

				if (JreturnValue != JFileChooser.APPROVE_OPTION)					
				{
					return; //cancel
				}
				else
				{
					files = Jfidi.getSelectedFiles();
				}
			}
			catch (Exception e)
			{}
			go.setEnabled(true);		
		}

		else if (ae.getSource() == openD)
		{	//folder mode
			mode = 1;
			go.setEnabled(false);
			convertDir = "";
			saveDir = "";

			JFileChooser Jfidi = new JFileChooser("");
			Jfidi.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 
			Jfidi.setDialogTitle("Select SOURCE Folder");
			int JreturnValue = Jfidi.showOpenDialog(new JFrame());
			
			if (JreturnValue != JFileChooser.APPROVE_OPTION)			
			{
				//error
			}
			else
			{
				if ( Jfidi.getCurrentDirectory().toString().endsWith(sep) )
					convertDir = Jfidi.getCurrentDirectory() + Jfidi.getSelectedFile().getName();
				else		
					convertDir = Jfidi.getCurrentDirectory() + sep + Jfidi.getSelectedFile().getName();
				//System.out.println(convertDir);

				JFileChooser Kfidi = new JFileChooser("");
				Kfidi.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 
				Kfidi.setDialogTitle("Select SAVE Folder");
				int KreturnValue = Kfidi.showOpenDialog(new JFrame());

				if (KreturnValue != JFileChooser.APPROVE_OPTION)			
				{
					//error
				}
				else
				{			
					if ( Kfidi.getCurrentDirectory().toString().endsWith(sep) )
						saveDir = Kfidi.getCurrentDirectory() + Kfidi.getSelectedFile().getName();
					else		
						saveDir = Kfidi.getCurrentDirectory() + sep + Kfidi.getSelectedFile().getName();
					//System.out.println(saveDir);

					if (convertDir.length() > 0 && saveDir.length() > 0)
						go.setEnabled(true);
				}
			}
		}

		if (ae.getSource() == go)
		{
			if (mode == 0)
			{
				for (int h = 0; h < files.length; h++)
				{
					filename = files[h].getName();
					changeVolume(files[h]);
				} 
			}
			else if (mode == 1)
			{
				String runname = "";
				File cd = new File(convertDir);
				File[] files = cd.listFiles();
				for (int i = 0; i < files.length; i++)
					if (files[i].isFile())
					{
						filename = files[i].getName();
						changeVolume(files[i]);
					}
			}
		}
	}

	private static void changeVolume(File stylefile)
	{
		if (up.isSelected())
			value = 100 + perc.getSelectedIndex();
		else 
			value = 100 - perc.getSelectedIndex();
	
		// read non MIDI
		content = new byte[(int)(stylefile.length())];
		try
		{
			FileInputStream infile = new FileInputStream(stylefile);
			boolean eof = false;
			int z = 0;
			while (!eof)
			{
				int input = infile.read();
				if (input == -1)
				{
					eof = true;
					break;
				}
				content[z] = (byte)input;

								// fast check
				if (z == 0 && content[z] != 77)
				{
					infile.close();
					return; 
				}
				else if (z == 1 && content[z] != 84)
				{
					infile.close();
					return; 
				}
				else if (z == 2 && content[z] != 104)
				{
					infile.close();
					return; 
				}
				else if (z == 3 && content[z] != 100)
				{
					infile.close();
					return; 
				}
				z++;
			}
			infile.close();
			infile = null;

			if (content.length < 100)
				return;

			m_end = 0;
			for (int j = 0; j < content.length - 2; j++)
			{
				//find midi part end = first "ff 2f 00"
				if (content[j] == (byte)255 && content[j+1] == 47 &&  content[j+2] == 0)
				{
					m_end = j+2;
					break;
				}
			}
			
			// new sequence
			Sequence style = MidiSystem.getSequence(stylefile);
			Track[] inTrack = style.getTracks();
			int fileRes = style.getResolution();

			newStyle = new Sequence(Sequence.PPQ, fileRes);
			Track newTrack  = newStyle.createTrack();

			for (int j = 0; j < inTrack[0].size(); j++)
			{
				MidiEvent event = inTrack[0].get(j);
				MidiMessage message = event.getMessage();
				byte[] mess = message.getMessage();
				long runTicks = event.getTick();
				int status = message.getStatus();

				if ( vol.isSelected() && status >= 176 && status < 192 && ((ShortMessage)message).getData1() == 7)
				{
					int newVol = (((ShortMessage)message).getData2() * value ) / 100 ;
					if (newVol > 127)
						newVol = 127;
					ShortMessage shortMes = new ShortMessage();
					shortMes.setMessage(status, ((ShortMessage)message).getData1(), newVol);
					newTrack.add(new MidiEvent(shortMes, runTicks));
				}
				else if ( vel.isSelected() && status >= 144 && status < 159)
				{
					int newVel = (((ShortMessage)message).getData2() * value ) / 100 ;
					if (newVel > 127)
						newVel = 127;
					ShortMessage shortMes = new ShortMessage();
					shortMes.setMessage(status, ((ShortMessage)message).getData1(), newVel);
					newTrack.add(new MidiEvent(shortMes, runTicks));
				}
				else
					newTrack.add(event);
			}
			File outfile = null;
			if (mode == 0)
			{
				File fr = new File("_"+filename);
				JFileChooser fido = new JFileChooser("");
				fido.setSelectedFile(fr);
				fido.setDialogTitle("Save Converted Style File");
				int KreturnValue = fido.showSaveDialog(new Frame());

				if (KreturnValue != JFileChooser.APPROVE_OPTION)			
					return;
				else
					outfile  = new File(fido.getCurrentDirectory()+ sep + fido.getSelectedFile().getName());	
				
				MidiSystem.write(newStyle, 0, outfile);
				FileOutputStream fos = new FileOutputStream(outfile, true);
				fos.write(content, m_end + 1, content.length - m_end - 1);
				fos.close();
				fos = null;			
			}
			else if (mode == 1)
			{
				outfile  = new File(saveDir, filename);
				MidiSystem.write(newStyle, 0, outfile);
				FileOutputStream fos = new FileOutputStream(outfile, true);
				fos.write(content, m_end + 1, content.length - m_end - 1);
				fos.close();
				fos = null;
			}
		}
		catch(Exception e)
		{}
	}
}